@@ -22,6 +22,7 @@
 #include <qclipboard.h>
 // lib specific
 // #include "khe.h"
+#include "khexedit_export.h"
 #include "kcolumnsview.h"
 
 class QTimer;
@@ -43,9 +44,16 @@ class KBufferCursor;
 class KBufferLayout;
 class KBufferRanges;
 
+class KController;
+class KTabController;
+class KNavigator;
+class KValueEditor;
+class KCharEditor;
+
 class KBufferDrag;
 
 class KCursor;
+class KCharCodec;
 
 class KHexEditPrivate;
 
@@ -64,8 +72,14 @@ class KHexEditPrivate;
   *@author Friedrich W. H. Kossebau
   */
 
-class KHexEdit : public KColumnsView
+class KHEXEDIT_EXPORT KHexEdit : public KColumnsView
 {
+  friend class KTabController;
+  friend class KNavigator;
+  friend class KEditor;
+  friend class KValueEditor;
+  friend class KCharEditor;
+
   Q_OBJECT
   Q_ENUMS( KResizeStyle KCoding )
   Q_PROPERTY( bool OverwriteMode READ isOverwriteMode WRITE setOverwriteMode )
@@ -94,14 +108,12 @@ class KHexEdit : public KColumnsView
   Q_PROPERTY( QChar SubstituteChar READ substituteChar WRITE setSubstituteChar )
 
   public:
-    enum KMoveAction { MoveBackward, MoveWordBackward, MoveForward, MoveWordForward,
-                       MoveUp, MovePgUp, MoveDown, MovePgDown,
-                       MoveLineStart, MoveHome, MoveLineEnd, MoveEnd };
-    enum KKeyboardAction { ActionDelete, ActionWordDelete, ActionBackspace, ActionWordBackspace };
-
     enum KResizeStyle { NoResize=0, LockGrouping=1, FullSizeUsage=2, MaxResizeStyleId=0xFF };
     enum KCoding { HexadecimalCoding=0, DecimalCoding=1, OctalCoding=2, BinaryCoding=3, MaxCodingId=0xFFFF };
-    enum KEncoding { LocalEncoding=0, ISO8859_1Encoding=1, MaxEncodingId=0xFFFF };
+    enum KEncoding { LocalEncoding=0, ISO8859_1Encoding=1, EBCDIC1047Encoding=2,
+                     StartOfOwnEncoding=0x8000, MaxEncodingId=0xFFFF };
+
+    enum KBufferColumnId { ValueColumnId=1, CharColumnId=2 };
 
 
   public:
@@ -122,8 +134,11 @@ class KHexEdit : public KColumnsView
 
 
   public: // value access
-    /** returns true if there is a selected range in the array */
-    bool hasSelectedData() const;
+    /** returns the index of the cursor position */
+    int cursorPosition() const;
+    /***/
+    bool isCursorBehind() const;
+    KBufferColumnId cursorColumn() const;
 
     bool isOverwriteMode() const;
     bool isOverwriteOnly() const;
@@ -136,14 +151,17 @@ class KHexEdit : public KColumnsView
     int noOfBytesPerLine() const;
     int startOffset() const;
     int firstLineOffset() const;
-    
+
+    bool offsetColumnVisible() const;
+    int visibleBufferColumns() const;
+
   // value column
     KCoding coding() const;
     int/*KPixelX*/ byteSpacingWidth() const;
     int noOfGroupedBytes() const;
     int/*KPixelX*/ groupSpacingWidth() const;
     int/*KPixelX*/ binaryGapWidth() const;
-    
+
   // char column
     /** reports if "unprintable" chars (value<32) are displayed in the char column
       * with their original character. Default is false
@@ -154,11 +172,16 @@ class KHexEdit : public KColumnsView
       * @return substitute character 
       */
     QChar substituteChar() const;
+    /** returns the actually used undefined character for "undefined" chars, default is '?' */
+    QChar undefinedChar() const;
     /**
       * @return encoding used in the char column
       */
     KEncoding encoding() const;
-
+    /**
+     * @return name of the encoding used in the char column
+     */
+    const QString &encodingName() const;
 
   public: // logic value service
     /** calculates the number of bytes per line that fit into a widget with the given size
@@ -175,20 +198,23 @@ class KHexEdit : public KColumnsView
     int indexByPoint(const QPoint &Point ) const;
 
   public:
+    /** returns true if there is a selected range in the array */
+    bool hasSelectedData() const;
     /** 
       * @return deep copy of the selected data 
       */
     QByteArray selectedData() const;
+    KSection selection() const;
 
   public: // modification access
-    /** moves the cursor according to the action, handles all drawing */
-    void moveCursor( KMoveAction Action );
     /** puts the cursor to the position of index, handles all drawing 
       * @param Index 
       */
-    void setCursorPosition( int Index );
+    void setCursorPosition( int Index, bool Behind=false );
     /** puts the cursor in the column at the pos of Point (in absolute coord), does not handle the drawing */
     void placeCursor( const QPoint &Point );
+    /***/
+    void setCursorColumn( KBufferColumnId );
 //    void repaintByte( int row, int column, bool Erase = true );
 //    void updateByte( int row, int column );
 //    void ensureByteVisible( int row, int column );
@@ -197,8 +223,10 @@ class KHexEdit : public KColumnsView
     /** */
     void setDataBuffer( KDataBuffer *B );
 
-//    /** switches the Offset column on/off */
-//    void toggleOffsetColumn( bool Visible );
+    /** switches the Offset column on/off */
+    void toggleOffsetColumn( bool Visible );
+    /** */
+    void showBufferColumns( int Columns );
     /** scrolls the view as much as needed to have the cursor fully visible */
     void ensureCursorVisible();
 
@@ -267,19 +295,26 @@ class KHexEdit : public KColumnsView
       * returns true if there was a change
       */
     void setSubstituteChar( QChar SC );
+    /** sets the undefined character for "undefined" chars
+     * returns true if there was a change
+     */
+    void setUndefinedChar( QChar UC );
     /** sets the encoding of the char column. Default is KHE::LocalEncoding.
       * If the encoding is not available the format will not be changed. */
     void setEncoding( KEncoding C );
+    /** sets the encoding of the char column. Default is KHE::LocalEncoding.
+      * If the encoding is not available the format will not be changed.
+      * @param Encoding name of the encoding
+      */
+    void setEncoding( const QString& Encoding );
 
   // interaction
     /** de-/selects all data */
     void selectAll( bool select );
+    /** de-/selects all data */
+    void select( KSection S );
     /** selects word at index, returns true if there is one */
     bool selectWord( /*unsigned*/ int Index /*, Chartype*/ );
-    /**  */
-    void moveCursor( KMoveAction Action, bool Select );
-    /** executes keyboard Action \a Action. This is normally called by a key event handler. */
-    void doKeyboardAction( KKeyboardAction Action );
     /** removes the selected data, takes care of the cursor */
     virtual void removeSelectedData();
     /** inserts */
@@ -308,17 +343,6 @@ class KHexEdit : public KColumnsView
     /** undoes pauseCursor */
     virtual void unpauseCursor();
 
-  // byte editing
-    /** steps inside editing the byte in the value column */
-    bool goInsideByte();
-    /** */
-    void goOutsideByte( bool MoveToNext = false );
-    /** increases the byte in the buffer */
-    bool incByte();
-    /** increases the byte in the buffer */
-    bool decByte();
-
-
 
   signals:
     /** Index of the byte that was clicked */
@@ -328,15 +352,13 @@ class KHexEdit : public KColumnsView
 
     void cursorPositionChanged( int Index );
     /** selection has changed */
-    void selectionChanged();
+    void selectionChanged( int StartIndex, int EndIndex );
     /** there is a cut available or not */
     void cutAvailable( bool Really );
     /** there is a copy available or not */
     void copyAvailable( bool Really );
     /** there has been a change to the buffer */
-    void bufferChanged();
-    /** the input was not even partially inserted */
-    void inputFailed();
+    void bufferChanged( int StartIndex, int EndIndex );
 
 
   protected: // QWidget API
@@ -359,27 +381,23 @@ class KHexEdit : public KColumnsView
   protected: // KColumnsView API
     virtual void setNoOfLines( int NewNoOfLines );
 
-  
+
   protected: // element accessor functions
-    KOffsetColumn& offsetColumn();
     KValueColumn& valueColumn();
     KCharColumn& charColumn();
     KBufferColumn& activeColumn();
     KBufferColumn& inactiveColumn();
-    const KOffsetColumn& offsetColumn() const;
     const KValueColumn& valueColumn()    const;
     const KCharColumn& charColumn()   const;
     const KBufferColumn& activeColumn() const;
     const KBufferColumn& inactiveColumn() const;
-  
+
   protected: // atomic ui operations
     /** handles screen update in case of a change to any of the width sizes
       */
     void updateViewByWidth();
     /** repaints all the parts that are signed as changed */
     void repaintChanged();
-    bool handleByteEditKey( QKeyEvent *KeyEvent );
-    bool handleLetter( QKeyEvent *KeyEvent );
 
   protected: // drawing related operations
     /** recreates the cursor pixmaps and paints active and inactive cursors if doable */
@@ -389,21 +407,18 @@ class KHexEdit : public KColumnsView
     /** draws the blinking cursor or removes it */
     void paintActiveCursor( bool CursorOn );
     void paintInactiveCursor( bool CursorOn );
-    void paintEditedByte( bool Edited );
     void paintLine( KBufferColumn *C, int Line, KSection Positions );
 
   protected: // partial operations
     void handleMouseMove( const QPoint& Point );
-    KBufferDrag *dragObject( bool F = false, QWidget *Parent = 0 ) const;
+    KBufferDrag *dragObject( QWidget *Parent = 0 ) const;
     void pasteFromSource( QMimeSource *Source );
     /** removes the section from the databuffer and updates all affected values */
-    void removeData( KSection Indizes );
+    KSection removeData( KSection Indizes );
     /** sets ChangedRange to the range of VisibleRange that is actually changed
       * @return true if there was a change within the visible range
       */
     bool hasChanged( const KCoordRange &VisibleRange, KCoordRange *ChangedRange ) const;
-    /** copies the actual edit value to the databuffer and updates the coding string */
-    void syncEditedByte();
     void handleInternalDrag( QDropEvent *e );
 
   protected:
@@ -418,7 +433,6 @@ class KHexEdit : public KColumnsView
     /** calls updateContent for the Column */
     void updateColumn( KColumn &Column );
 
-
   protected slots:
     /** gets called by the cursor blink timer */
     void blinkCursor();
@@ -444,11 +458,11 @@ class KHexEdit : public KColumnsView
     /** */
     KBufferRanges *BufferRanges;
 
-    
+
   protected:
     KOffsetColumn *OffsetColumn;
     KBorderColumn *FirstBorderColumn;
-    KValueColumn    *ValueColumn;
+    KValueColumn  *ValueColumn;
     KBorderColumn *SecondBorderColumn;
     KCharColumn   *CharColumn;
 
@@ -457,6 +471,17 @@ class KHexEdit : public KColumnsView
     /** points to the column without keyboard focus (if there is) */
     KBufferColumn *InactiveColumn;
 
+    /** the actual input controller */
+    KController *Controller;
+    /** */
+    KTabController *TabController;
+    /** */
+    KNavigator *Navigator;
+    /** */
+    KValueEditor *ValueEditor;
+    /** */
+    KCharEditor *CharEditor;
+
   protected:
     /** Timer that controls the blinking of the cursor */
     QTimer *CursorBlinkTimer;
@@ -470,14 +495,8 @@ class KHexEdit : public KColumnsView
 
     /** object to store the blinking cursor pixmaps */
     KCursor *CursorPixmaps;
-    /** buffer with the  */
-    char *ByteBuffer;
-    /** stores the number of actual digits */
-    int LengthOfByteBuffer;
     /** */
-    unsigned char EditValue;
-    /** stores the old byte value */
-    unsigned char OldValue;
+    KCharCodec *Codec;
 
   protected:
     /** point at which the current double click happended (used by TrippleClick) */
@@ -494,8 +513,9 @@ class KHexEdit : public KColumnsView
   protected: // parameters
     /** style of resizing */
     KResizeStyle ResizeStyle;
-    /** flag if tab key should be ignored */
-    bool TabChangesFocus:1;
+    /** */
+    KEncoding Encoding;
+
     /** flag whether the widget is set to readonly. Cannot override the databuffer's setting, of course. */
     bool ReadOnly:1;
     /** flag if only overwrite is allowed */
@@ -516,10 +536,6 @@ class KHexEdit : public KColumnsView
     bool BlinkCursorVisible:1;
     /** flag whether the font is changed due to a zooming */
     bool InZooming:1;
-    /** flag whether we are in byte digits editing mode */
-    bool InEditMode:1;
-    /** flag whether byte edit mode was reached by inserting */
-    bool EditModeByInsert:1;
 
   private:
     /** the binary compatibility saving helper */
@@ -531,13 +547,11 @@ class KHexEdit : public KColumnsView
 };
 
 
-inline const KOffsetColumn& KHexEdit::offsetColumn()   const { return *OffsetColumn; }
 inline const KValueColumn& KHexEdit::valueColumn()     const { return *ValueColumn; }
 inline const KCharColumn& KHexEdit::charColumn()       const { return *CharColumn; }
 inline const KBufferColumn& KHexEdit::activeColumn()   const { return *ActiveColumn; }
 inline const KBufferColumn& KHexEdit::inactiveColumn() const { return *InactiveColumn; }
 
-inline KOffsetColumn& KHexEdit::offsetColumn()   { return *OffsetColumn; }
 inline KValueColumn& KHexEdit::valueColumn()     { return *ValueColumn; }
 inline KCharColumn& KHexEdit::charColumn()       { return *CharColumn; }
 inline KBufferColumn& KHexEdit::activeColumn()   { return *ActiveColumn; }
